local Enemy = Enemy
local g = {}

local YOUR_HITS = SystemData.ChatLogFilters.YOUR_HITS
local YOUR_HEALS = SystemData.ChatLogFilters.YOUR_HEALS
--local PET_DMG = SystemData.ChatLogFilters.PET_DMG
local PET_HITS = SystemData.ChatLogFilters.PET_HITS
local YOUR_DMG_FROM_PC = SystemData.ChatLogFilters.YOUR_DMG_FROM_PC
local YOUR_DMG_FROM_NPC = SystemData.ChatLogFilters.YOUR_DMG_FROM_NPC

Enemy.CombatLogTypes =
{
	IncommingDamage = 1,
	OutgoingDamage = 2,
	IncommingHeal = 3,
	OutgoingHeal = 4
}

local pairs = pairs
local ipairs = ipairs
local tonumber = tonumber
local strmatch = wstring.match
local tinsert = table.insert
local tremove = table.remove
local mmin = math.min
local mmax = math.max


--------------------------------------------------------------- Init

function Enemy.CombatLogInitialize ()

	Enemy.combatLog = g
	g.isPluginEnabled = Enemy.DefaultSettings.combatLogEnabled
		
	g.allStats = Enemy.Settings.CombatLogStats
	if (not g.allStats) then Enemy.CombatLog_ResetStatsAllSessions () end
	
	g.stats = g.allStats[#g.allStats]
	if (not g.stats) then Enemy.CombatLog_StatsSessionAdd () end
	
	--g.currentTargetNames = {}
	--Enemy.CombatLog_ResetCurrentStats ()
	
	-- UI: config
	g.configDlgSection =
	{
		name = "CombatLog",
		title = L"Combat log",
		templateName = "EnemyCombatLogConfiguration",
		onInitialize = Enemy.CombatLogUI_ConfigDialog_OnInitialize,
		onLoad = Enemy.CombatLogUI_ConfigDialog_OnLoad,
		onSave = Enemy.CombatLogUI_ConfigDialog_OnSave,
		onReset = Enemy.CombatLogUI_ConfigDialog_OnReset
	}
	
	Enemy.CombatLogUI_StatsWindow_Initialize ()
	Enemy.CombatLogUI_IDS_Initialize ()
	
	-- static events
	Enemy.AddEventHandler ("CombatLog", "SettingsChanged", Enemy.CombatLog_OnSettingsChanged)
	
	Enemy.AddEventHandler ("CombatLog", "ConfigDialogInitializeSections", function (sections)
		table.insert (sections, g.configDlgSection)
	end)
	
	Enemy.TriggerEvent ("CombatLogInitialized")
end


function Enemy._CombatLogEnabledChanged (enable)

	if (g.isPluginEnabled == enable) then return end
	g.isPluginEnabled = enable
	
	if (g.isPluginEnabled)
	then
		Enemy.CombatLogInitLocalization ()
		
		g.strCrit = Enemy.Localize("combatLogParse_strCrit")
		g.strBlock = Enemy.Localize("combatLogParse_strBlock")
		g.strParry = Enemy.Localize("combatLogParse_strParry")
		g.strDodge = Enemy.Localize("combatLogParse_strDodge")
		g.strDisrupted = Enemy.Localize("combatLogParse_strDisrupted")
		g.strYou = Enemy.Localize("combatLogParse_strYou")
		g.regexMitigation = Enemy.Localize("combatLogParse_regexMitigation")
		g.regexAbsorb = Enemy.Localize("combatLogParse_regexAbsorb")
		g.regexOverheal = Enemy.Localize("combatLogParse_regexOverheal")
		g.regexIncommingDamage = Enemy.Localize("combatLogParse_regexIncommingDamage")
		g.regexIncommingDamageIgnored = Enemy.Localize("combatLogParse_regexIncommingDamageIgnored")
		g.regexOutgoingDamage = Enemy.Localize("combatLogParse_regexOutgoingDamage")
		g.regexOutgoingDamageIgnored = Enemy.Localize("combatLogParse_regexOutgoingDamageIgnored")
		g.regexOutgoingPetDamage = Enemy.Localize("combatLogParse_regexOutgoingPetDamage")
		g.regexOutgoingPetDamageIgnored = Enemy.Localize("combatLogParse_regexOutgoingPetDamageIgnored")
		g.regexOutgoingHeal = Enemy.Localize("combatLogParse_regexOutgoingHeal")
		g.regexIncommingHeal = Enemy.Localize("combatLogParse_regexIncommingHeal")
		g.regexIncommingHealYours = Enemy.Localize("combatLogParse_regexIncommingHealYours")
		
		g.damageIgnoreTypeByAction =
		{
			[g.strBlock] = "block",
			[g.strParry] = "parry",
			[g.strDodge] = "dodge",
			[g.strDisrupted] = "disrupt"
		}
		
		g.combatLogParseByType =
		{
			[YOUR_DMG_FROM_NPC] = Enemy.CombatLog_ParseIncommingDamage,
			[YOUR_DMG_FROM_PC] = Enemy.CombatLog_ParseIncommingDamage,
			[YOUR_HITS] = Enemy.CombatLog_ParseOutgoingDamage,
			[PET_HITS] = Enemy.CombatLog_ParseOutgoingPetDamage,
			[YOUR_HEALS] = Enemy.CombatLog_ParseHeal
		}
		
		Enemy.CombatLogFixLocalization (g)
		
		RegisterEventHandler (TextLogGetUpdateEventId ("Combat"), "Enemy.CombatLog_OnCombatLog")
		--Enemy.AddEventHandler ("CombatLog", "TargetChanged", Enemy.CombatLog_OnTargetChanged)
	else
		UnregisterEventHandler (TextLogGetUpdateEventId ("Combat"), "Enemy.CombatLog_OnCombatLog")
		--Enemy.RemoveEventHandler ("CombatLog", "TargetChanged")
	end
end


function Enemy.CombatLog_OnSettingsChanged (settings)

	g.settings = settings
	Enemy._CombatLogEnabledChanged (g.settings.combatLogEnabled)
	Enemy.TriggerEvent ("CombatLogSettingsChanged", g.settings)
end



--------------------------------------------------------------- Sessions

function Enemy.CombatLog_ResetStatsAllSessions ()
	
	g.allStats = {}
	Enemy.Settings.CombatLogStats = g.allStats

	Enemy.CombatLog_StatsSessionAdd ()	
	Enemy.CombatLog_ResetStatsCurrentSession ()
end


function Enemy.CombatLog_ResetStatsCurrentSession ()

	g.stats.dt = Enemy.GetCurrentDateTime ()
	g.stats.data = {}
	
	Enemy.TriggerEvent ("CombatLogSessionsUpdated")
end


function Enemy.CombatLog_StatsSessionAdd (name)

	g.stats =
	{
		name = name or L"Default",
		dt = Enemy.GetCurrentDateTime (),
		data = {}
	}
	
	tinsert (g.allStats, g.stats)
	Enemy.TriggerEvent ("CombatLogSessionsUpdated")
end


function Enemy.CombatLog_StatsSessionDelete (indexOrStatsObject)

	if (not indexOrStatsObject) then return end
	
	local index = indexOrStatsObject

	if (type (indexOrStatsObject) == "table")
	then
		index = nil
		
		for k, v in ipairs (g.allStats)
		do
			d (Enemy.toWStringOrEmpty (indexOrStatsObject), Enemy.toWStringOrEmpty (v))
			
			if (v == indexOrStatsObject)
			then
				index = k
				break
			end
		end
	end
	
	if (not index) then return end	
	tremove (g.allStats, index)
	
	if (#g.allStats < 1)
	then
		Enemy.CombatLog_StatsSessionAdd ()
	end
	
	Enemy.TriggerEvent ("CombatLogSessionsUpdated")
end



--------------------------------------------------------------- Main functionality


--function Enemy.CombatLog_ResetCurrentStats ()
--
--	g.currentStats = {}
--
--	for _, t in pairs (Enemy.CombatLogTypes)
--	do
--		-- two values: one for current target and one for all targets affected (aoe)
--		local stats =
--		{
--			damage = 0,
--			damageAoe = 0,
--			
--			heal = 0,
--			healAoe = 0,
--			
--			overheal = 0,
--			overhealAoe = 0,
--			
--			mit = 0,
--			mitAoe = 0,
--			abs = 0,
--			absAoe = 0,
--			
--			block = 0,
--			blockAoe = 0,
--			parry = 0,
--			parryAoe = 0,
--			disrupt = 0,
--			disruptAoe = 0,
--			dodge = 0,
--			dodgeAoe = 0,
--			
--			data = EnemyLinkedList.New ()
--		}
--		
--		g.currentStats[t] = stats
--	end
--end
--
--
--function Enemy._CombatLog_ResetCurrentStatsNonAoe (stats)
--	stats.damage = 0
--	stats.heal = 0
--	stats.overheal = 0
--	stats.mit = 0
--	stats.abs = 0
--	stats.block = 0
--	stats.parry = 0
--	stats.disrupt = 0
--	stats.dodge = 0
--end


function Enemy.CombatLog_OnCombatLog (updateType, filterType)

	if (updateType ~= SystemData.TextLogUpdate.ADDED
		or
		not g.combatLogParseByType[filterType])
	then
		return
	end
	
	local time, t, msg = TextLogGetEntry ("Combat", TextLogGetNumEntries ("Combat") - 1)
	if (not (msg)) then return end
	
	local combat_event = g.combatLogParseByType[t] (msg, Enemy.CombatLog_ParseTime (time))
	
	if (not combat_event and g.settings.combatLogLogParseErrors)
	then
		d (L"Error parsing ["..Enemy.toWString (t)..L"] "..msg)
		return
	end

	-- combat statistics
	local combat_event2 = nil
	if (g.settings.combatLogStatisticsEnabled)
	then
		Enemy._CombatLog_StatisticsAddCombatEvent (combat_event)
		
		-- if we've been healing ourself then emulate incomming heal combat event
		if (combat_event.type == Enemy.CombatLogTypes.OutgoingHeal and combat_event.object == g.strYou)
		then
			combat_event2 = Enemy.clone (combat_event)
			combat_event2.type = Enemy.CombatLogTypes.IncommingHeal
			combat_event2.object = Enemy.playerName
			
			Enemy._CombatLog_StatisticsAddCombatEvent (combat_event2)
		end
	end
	
	-- current statistics
--	if (Enemy.Settings.combatLogCurrentStatisticsEnabled[combat_event.type])
--	then
--		local stats = g.currentStats[combat_event.type]
--		
--		stats.data:Add (combat_event)
--		
--		-- add statistical data
--		Enemy._CombatLog_CurrentStatisticsAddCombatEvent (combat_event, stats)
--		
--		-- remove all combat events that exceeds measuring timeframe interval 
--		local max_time = Enemy.time - g.settings.combatLogCurrentStatisticsTimeframe
--		
--		local tmp = stats.data.first
--		while (tmp and tmp.data.time < max_time)
--		do
--			local old_combat_event = tmp.data
--		
--			-- remove statistical data
--			if (old_combat_event.overheal == nil)
--			then
--				-- damage
--				local target = Enemy.latestTargets[TargetInfo.HOSTILE_TARGET]
--				
--				if (target and target.name == old_combat_event.object)
--				then
--					if (old_combat_event.damageIgnoreType == nil)
--					then
--						stats.damage = stats.damage - old_combat_event.value
--						stats.mit = stats.mit - old_combat_event.mit
--						stats.abs = stats.abs - old_combat_event.abs
--					else
--						local n = old_combat_event.damageIgnoreType
--						stats[n] = stats[n] - 1
--					end
--				end
--				
--				if (old_combat_event.damageIgnoreType == nil)
--				then
--					stats.damageAoe = stats.damageAoe - old_combat_event.value
--					stats.mitAoe = stats.mitAoe - old_combat_event.mit
--					stats.absAoe = stats.absAoe - old_combat_event.abs
--				else
--					local n = old_combat_event.damageIgnoreType.."Aoe"
--					stats[n] = stats[n] - 1
--				end
--			else
--				-- heal
--				local target = Enemy.latestTargets[TargetInfo.FRIENDLY_TARGET]
--				
--				if (target and target.name == old_combat_event.object)
--				then
--					stats.heal = stats.heal - old_combat_event.value
--					stats.overheal = stats.overheal - old_combat_event.overheal
--				end
--				
--				stats.healAoe = stats.healAoe - old_combat_event.value
--				stats.overhealAoe = stats.overhealAoe - old_combat_event.overheal
--			end
--		
--			stats.data:Remove (tmp)
--			tmp = stats.data.first
--		end
--	end

--	TODO
--	if (not stats.data.first) ...

	Enemy.TriggerEvent ("CombatLogNewCombatEvent", combat_event)
	if (combat_event2) then Enemy.TriggerEvent ("CombatLogNewCombatEvent", combat_event2) end
end


function Enemy._CombatLog_StatisticsAddCombatEvent (combatEvent)

	local stats = g.stats.data[combatEvent.type]
	if (not stats)
	then
		stats = {}
		g.stats.data[combatEvent.type] = stats
	end
	
	local ability_name = combatEvent.ability
	if (combatEvent.type == Enemy.CombatLogTypes.IncommingHeal)
	then
		ability_name = combatEvent.object..L" - "..ability_name
	end
	
	local ability_stats = stats[ability_name]
	if (not ability_stats)
	then
		ability_stats =
		{
			countNormal = 0,
			countCrits = 0,
			
			totalNormal = 0,
			totalCrits = 0,
			totalOverheal = 0,
			
			mit = 0,
			abs = 0,
			
			min = 0,
			max = 0,
			
			block = 0,
			parry = 0,
			disrupt = 0,
			dodge = 0
		}
		
		stats[ability_name] = ability_stats
	end
	
	if (combatEvent.damageIgnoreType == nil)
	then
		if (not combatEvent.crit)
		then
			ability_stats.countNormal = ability_stats.countNormal + 1
			ability_stats.totalNormal = ability_stats.totalNormal + combatEvent.value
		else
			ability_stats.countCrits = ability_stats.countCrits + 1
			ability_stats.totalCrits = ability_stats.totalCrits + combatEvent.value
		end
		
		if (combatEvent.overheal == nil)
		then			
			ability_stats.mit = ability_stats.mit + combatEvent.mit
			ability_stats.abs = ability_stats.abs + combatEvent.abs
		end
		
		ability_stats.min = mmin(ability_stats.min, combatEvent.value)
		ability_stats.max = mmax(ability_stats.max, combatEvent.value)
		
	else
		ability_stats.countNormal = ability_stats.countNormal + 1
		
		local n = combatEvent.damageIgnoreType
		ability_stats[n] = ability_stats[n] + 1
	end
	
	if (combatEvent.overheal ~= nil)
	then
		ability_stats.totalOverheal = ability_stats.totalOverheal + combatEvent.overheal
	end
end


--function Enemy._CombatLog_CurrentStatisticsAddCombatEvent (combatEvent, stats)
--
--	if (combatEvent.overheal == nil)
--	then
--		-- damage
--		local target = Enemy.latestTargets[TargetInfo.HOSTILE_TARGET]
--		
--		if (target and target.name == combatEvent.object)
--		then
--			if (combatEvent.damageIgnoreType == nil)
--			then
--				stats.damage = stats.damage + combatEvent.value
--				stats.mit = stats.mit + combatEvent.mit
--				stats.abs = stats.abs + combatEvent.abs
--			else
--				local n = combatEvent.damageIgnoreType
--				stats[n] = stats[n] + 1
--			end
--		end
--		
--		if (combatEvent.damageIgnoreType == nil)
--		then
--			stats.damageAoe = stats.damageAoe + combatEvent.value
--			stats.mitAoe = stats.mitAoe + combatEvent.mit
--			stats.absAoe = stats.absAoe + combatEvent.abs
--		else
--			local n = combatEvent.damageIgnoreType.."Aoe"
--			stats[n] = stats[n] + 1
--		end
--	else
--		-- heal
--		local target = Enemy.latestTargets[TargetInfo.FRIENDLY_TARGET]
--		
--		if (target and target.name == combatEvent.object)
--		then
--			stats.heal = stats.heal + combatEvent.value
--			stats.overheal = stats.overheal + combatEvent.overheal
--		end
--		
--		stats.healAoe = stats.healAoe + combatEvent.value
--		stats.overhealAoe = stats.overhealAoe + combatEvent.overheal
--	end
--end


--function Enemy.CombatLog_OnTargetChanged (type, target)
--
--	target = target or {}
--
--	if (target.name ~= g.currentTargetNames[type])
--	then
--		-- the target has been changed
--		-- we need to recalculate all current target (non aoe) statistics
--		-- based on combat event history stored in "data" field
--		
--		g.currentTargetNames[type] = target.name
--		
--		local types
--		
--		-- determine stats type affected
--		if (type == TargetInfo.HOSTILE_TARGET)
--		then
--			types = { Enemy.CombatLogTypes.OutgoingDamage, Enemy.CombatLogTypes.IncommingDamage }
--		else
--			types = { Enemy.CombatLogTypes.OutgoingHeal, Enemy.CombatLogTypes.IncommingHeal }
--		end
--		
--		-- recalculate
--		for _, t in ipairs (types)
--		do
--			local stats = g.currentStats[t]
--			Enemy._CombatLog_ResetCurrentStatsNonAoe (stats)
--			
--			if (target.name)
--			then			
--				local e = stats.data.first
--				while (e)
--				do
--					Enemy._CombatLog_CurrentStatisticsAddCombatEvent (e.data, stats)
--					e = e.next
--				end
--			end
--		end
--	end
--end


--------------------------------------------------------------- Parse function

function Enemy.CombatLog_ParseTime (str)

	if (not str) then return nil end

	local h, m, s = strmatch (str, L"^%[(%d%d):(%d%d):(%d%d)%]")
	return tonumber (h) * 3600 + tonumber (m) * 60 + tonumber (s)
end


function Enemy.CombatLog_ParseIncommingDamage (str, time)

	local object, ability, crit, val, mit = strmatch (str, g.regexIncommingDamage)

	if (object)
	then
		return
		{
			time = time,
			type = Enemy.CombatLogTypes.IncommingDamage,
			str = str,
			
			object = object,
			ability = ability,
			crit = (crit == g.strCrit),
			value = tonumber (val or 0),
			mit = tonumber (strmatch (mit, g.regexMitigation) or 0),
			abs = tonumber (strmatch (mit, g.regexAbsorb) or 0)
		}
	else
		local action, object, ability = strmatch (str, g.regexIncommingDamageIgnored)
		local damage_ignore_type = g.damageIgnoreTypeByAction[action]
		
		if (damage_ignore_type)
		then		
			return
			{
				time = time,
				type = Enemy.CombatLogTypes.IncommingDamage,
				str = str,
				
				object = object,
				ability = ability,
				
				damageIgnoreType = damage_ignore_type
			}
		end
	end
	
	return nil
end


function Enemy.CombatLog_ParseOutgoingDamage (str, time)

	local ability, crit, object, val, mit = strmatch (str, g.regexOutgoingDamage)

	if (object)
	then
		return
		{
			time = time,
			type = Enemy.CombatLogTypes.OutgoingDamage,
			str = str,
			
			object = object,
			ability = ability,
			crit = (crit == g.strCrit),
			value = tonumber (val or 0),
			mit = tonumber (strmatch (mit, g.regexMitigation) or 0),
			abs = tonumber (strmatch (mit, g.regexAbsorb) or 0)
		}
	else
		local object, action, ability = strmatch (str, g.regexOutgoingDamageIgnored)
		local damage_ignore_type = g.damageIgnoreTypeByAction[action]
		
		if (action)
		then		
			return
			{
				time = time,
				type = Enemy.CombatLogTypes.OutgoingDamage,
				str = str,
				
				object = object,
				ability = ability,
				
				damageIgnoreType = damage_ignore_type
			}
		end
	end
	
	return nil
end


function Enemy.CombatLog_ParseOutgoingPetDamage (str, time)

	local petname, ability, crit, object, val, mit = strmatch (str, g.regexOutgoingPetDamage)

	if (object)
	then
		return
		{
			time = time,
			type = Enemy.CombatLogTypes.OutgoingDamage,
			pet = true,
			str = str,
			
			object = object,
			ability = ability,
			crit = (crit == g.strCrit),
			value = tonumber (val or 0),
			mit = tonumber (strmatch (mit, g.regexMitigation) or 0),
			abs = tonumber (strmatch (mit, g.regexAbsorb) or 0)
		}
	else
		local object, action, petname, ability = strmatch (str, g.regexOutgoingPetDamageIgnored)
		local damage_ignore_type = g.damageIgnoreTypeByAction[action]
		
		if (action)
		then
			return
			{
				time = time,
				type = Enemy.CombatLogTypes.OutgoingDamage,
				pet = true,
				str = str,
				
				object = object,
				ability = ability,
				
				damageIgnoreType = damage_ignore_type
			}
		end
		
		-- Mythic in their infinite wisdom treats player healing his pet as pet hits :|
		return Enemy.CombatLog_ParseHeal (str, time)
	end
end



function Enemy.CombatLog_ParseHeal (str, time)

	local ability, crit, object, val, overheal = strmatch (str, g.regexIncommingHealYours)
	
	if (object)
	then
		return
		{
			time = time,
			type = Enemy.CombatLogTypes.OutgoingHeal,
			str = str,
			
			object = object,
			ability = ability,
			crit = (crit == g.strCrit),
			value = tonumber (val or 0),
			overheal = tonumber (strmatch (overheal, g.regexOverheal) or 0)
		}
	else
		object, ability, crit, val, overheal = strmatch (str, g.regexIncommingHeal)
		
		if (object)
		then
			return
			{
				time = time,
				type = Enemy.CombatLogTypes.IncommingHeal,
				str = str,
				
				object = object,
				ability = ability,
				crit = (crit == g.strCrit),
				value = tonumber (val or 0),
				overheal = tonumber (strmatch (overheal, g.regexOverheal) or 0)
			}
		else
			local ability, crit, object, val, overheal = strmatch (str, g.regexOutgoingHeal)
		
			if (object)
			then
				return
				{
					time = time,
					type = Enemy.CombatLogTypes.OutgoingHeal,
					str = str,
					
					object = object,
					ability = ability,
					crit = (crit == g.strCrit),
					value = tonumber (val or 0),
					overheal = tonumber (strmatch (overheal, g.regexOverheal) or 0)
				}
			end
		end
	end
	
	return nil
end


--------------------------------------------------------------- UI: Configuration
local config_dlg = {}

config_dlg.properties =
{
	combatLogEnabled =
	{
		key = "combatLogEnabled",
		order = 10,
		name = L"Enable combat log parsing",
		tooltip = L"Enable whole combat log parsing module",
		type = "bool",
		default = Enemy.DefaultSettings.combatLogEnabled
	},
	
	combatLogLogParseErrors =
	{
		key = "combatLogLogParseErrors",
		order = 20,
		name = L"Log parse errors to /debug log",
		tooltip = L"Enable this if you want to identify problems with combat log parsing. Type /debug and click on 'Logs' button to switch loggin on. Then close debug window. Only after that type /reloadui. After this all log messages will be stored in 'warhammer_directory/logs/uilog.log' file.",
		type = "bool",
		default = Enemy.DefaultSettings.combatLogLogParseErrors
	},
	
	combatLogStatisticsEnabled =
	{
		key = "combatLogStatisticsEnabled",
		order = 30,
		name = L"Enable collecting statistics",
		tooltip = L"This will enable gathering statistical data about combat log event like max damage for abilities, crit %, mitigation %, ect.",
		type = "bool",
		default = Enemy.DefaultSettings.combatLogStatisticsEnabled
	},
	
	
--	combatLogCurrentStatisticsTitle =
--	{
--		key = "combatLogCurrentStatisticsTitle",
--		order = 35,
--		name = L"Current statistics",
--		type = "title",
--		paddingTop = 40
--	},
--	
--	combatLogCurrentOutgoingDamageStatisticsEnabled =
--	{
--		key = "combatLogCurrentOutgoingDamageStatisticsEnabled",
--		order = 40,
--		name = L"Enable outgoing damage statistics",
--		tooltip = L"This will enable gathering statistical data about current outgoing damage",
--		type = "bool",
--		default = Enemy.DefaultSettings.combatLogCurrentStatisticsEnabled[Enemy.CombatLogTypes.OutgoingDamage]
--	},
--	
--	combatLogCurrentOutgoingHealStatisticsEnabled =
--	{
--		key = "combatLogCurrentOutgoingHealStatisticsEnabled",
--		order = 50,
--		name = L"Enable outgoing healing statistics",
--		tooltip = L"This will enable gathering statistical data about current outgoing healing",
--		type = "bool",
--		default = Enemy.DefaultSettings.combatLogCurrentStatisticsEnabled[Enemy.CombatLogTypes.OutgoingHeal]
--	},
--	
--	combatLogCurrentIncommingDamageStatisticsEnabled =
--	{
--		key = "combatLogCurrentIncommingDamageStatisticsEnabled",
--		order = 60,
--		name = L"Enable incomming damage statistics",
--		tooltip = L"This will enable gathering statistical data about current incomming damage",
--		type = "bool",
--		default = Enemy.DefaultSettings.combatLogCurrentStatisticsEnabled[Enemy.CombatLogTypes.IncommingDamage]
--	},
--	
--	combatLogCurrentIncommingHealStatisticsEnabled =
--	{
--		key = "combatLogCurrentIncommingHealStatisticsEnabled",
--		order = 70,
--		name = L"Enable incomming healing statistics",
--		tooltip = L"This will enable gathering statistical data about current incomming healing",
--		type = "bool",
--		default = Enemy.DefaultSettings.combatLogCurrentStatisticsEnabled[Enemy.CombatLogTypes.IncommingHeal]
--	},
--	
--	combatLogCurrentStatisticsTimeframe =
--	{
--		key = "combatLogCurrentStatisticsTimeframe",
--		order = 80,
--		name = L"Timeframe",
--		tooltip = L"Sets based on how many last seconds of combat log the statistics will be calculated",
--		type = "float",
--		min = 0.1,
--		max = 60,
--		default = Enemy.DefaultSettings.combatLogCurrentStatisticsTimeframe
--	},

	combatLogIDSTitle =
	{
		key = "combatLogIDSTitle",
		order = 100,
		name = L"Incomming damage statistics",
		type = "title",
		paddingTop = 40
	},
	combatLogIDSEnabled =
	{
		key = "combatLogIDSEnabled",
		order = 110,
		name = L"Enable incomming damage statistics",
		tooltip = L"Enable this to see window with list of players that currently hitting you",
		type = "bool",
		default = Enemy.DefaultSettings.combatLogIDSEnabled
	},
	combatLogIDSType =
	{
		key = "combatLogIDSType",
		order = 120,
		name = L"Display",
		tooltip = L"Sort and display type",
		type = "select",
		values =
		{
			{ text = L"Damage", value = "damage" },
			{ text = L"DPS", value = "dps" }
		},
		default = Enemy.DefaultSettings.combatLogIDSType
	},
	combatLogIDSTimeframe =
	{
		key = "combatLogIDSTimeframe",
		order = 130,
		name = L"Timeframe",
		tooltip = L"Sets based on how many last seconds of combat log the incomming damage statistics will be calculated",
		type = "float",
		min = 0.1,
		max = 60,
		default = Enemy.DefaultSettings.combatLogIDSTimeframe
	},
	combatLogIDSMaxRows =
	{
		key = "combatLogIDSMaxRows",
		order = 140,
		name = L"Max rows",
		tooltip = L"How many rows to display",
		type = "number",
		min = 1,
		max = 10,
		default = Enemy.DefaultSettings.combatLogIDSMaxRows
	},
	combatLogIDSDisplayTime =
	{
		key = "combatLogIDSDisplayTime",
		order = 150,
		name = L"Display time",
		tooltip = L"Amount of seconds to display rows after last stats update",
		type = "float",
		min = 0.1,
		max = 600,
		default = Enemy.DefaultSettings.combatLogIDSDisplayTime
	},
	combatLogIDSRowSize =
	{
		key = "combatLogIDSRowSize",
		order = 160,
		name = L"Size (XY)",
		tooltip = L"Row size",
		type = "int[]",
		size = 2,
		min = { 1, 1 },
		max = { 500, 500 },
		default = Enemy.DefaultSettings.combatLogIDSRowSize
	},
	combatLogIDSRowPadding =
	{
		key = "combatLogIDSRowPadding",
		order = 170,
		name = L"Row padding",
		tooltip = L"Space between rows",
		type = "number",
		min = 0,
		max = 100,
		default = Enemy.DefaultSettings.combatLogIDSRowPadding
	},
	combatLogIDSRowScale =
	{
		key = "combatLogIDSRowScale",
		order = 180,
		name = L"Row scaling",
		type = "float",
		min = 0.1,
		max = 5,
		default = Enemy.DefaultSettings.combatLogIDSRowScale
	},
	combatLogIDSRowBackground =
	{
		key = "combatLogIDSRowBackground",
		order = 190,
		name = L"Row background color",
		type = "color",
		default = Enemy.DefaultSettings.combatLogIDSRowBackground
	},
	combatLogIDSRowBackgroundAlpha =
	{
		key = "combatLogIDSRowBackgroundAlpha",
		order = 200,
		name = L"Row background opacity",
		type = "float",
		min = 0,
		max = 1,
		default = Enemy.DefaultSettings.combatLogIDSRowBackgroundAlpha
	},
}


function Enemy.CombatLogUI_ConfigDialog_OnInitialize (section)

	config_dlg.section = section
	
	local root = config_dlg.section.windowName.."ContentScrollChild"
	config_dlg.cwn = root.."Config"
	Enemy.CreateConfigurationWindow (config_dlg.cwn, root, config_dlg.properties, Enemy.CombatLogUI_ConfigDialog_TestSettings)
	
	ScrollWindowUpdateScrollRect (config_dlg.section.windowName.."Content")
end


function Enemy.CombatLogUI_ConfigDialog_OnLoad (section)

	config_dlg.isLoading = true
	
	config_dlg.combatLogEnabled = Enemy.Settings.combatLogEnabled
	config_dlg.combatLogLogParseErrors = Enemy.Settings.combatLogLogParseErrors
	config_dlg.combatLogStatisticsEnabled = Enemy.Settings.combatLogStatisticsEnabled
	
--	config_dlg.combatLogCurrentOutgoingDamageStatisticsEnabled = Enemy.Settings.combatLogCurrentStatisticsEnabled[Enemy.CombatLogTypes.OutgoingDamage]
--	config_dlg.combatLogCurrentOutgoingHealStatisticsEnabled = Enemy.Settings.combatLogCurrentStatisticsEnabled[Enemy.CombatLogTypes.OutgoingHeal]
--	config_dlg.combatLogCurrentIncommingDamageStatisticsEnabled = Enemy.Settings.combatLogCurrentStatisticsEnabled[Enemy.CombatLogTypes.IncommingDamage]
--	config_dlg.combatLogCurrentIncommingHealStatisticsEnabled = Enemy.Settings.combatLogCurrentStatisticsEnabled[Enemy.CombatLogTypes.IncommingHeal]
--	config_dlg.combatLogCurrentStatisticsTimeframe = Enemy.Settings.combatLogCurrentStatisticsTimeframe

	config_dlg.combatLogIDSEnabled = Enemy.Settings.combatLogIDSEnabled
	config_dlg.combatLogIDSType = Enemy.Settings.combatLogIDSType
	config_dlg.combatLogIDSTimeframe = Enemy.Settings.combatLogIDSTimeframe
	config_dlg.combatLogIDSMaxRows = Enemy.Settings.combatLogIDSMaxRows
	config_dlg.combatLogIDSDisplayTime = Enemy.Settings.combatLogIDSDisplayTime
	config_dlg.combatLogIDSRowSize = Enemy.clone (Enemy.Settings.combatLogIDSRowSize)
	config_dlg.combatLogIDSRowPadding = Enemy.Settings.combatLogIDSRowPadding
	config_dlg.combatLogIDSRowScale = Enemy.Settings.combatLogIDSRowScale
	config_dlg.combatLogIDSRowBackground = Enemy.clone (Enemy.Settings.combatLogIDSRowBackground)
	config_dlg.combatLogIDSRowBackgroundAlpha = Enemy.Settings.combatLogIDSRowBackgroundAlpha
	
	Enemy.ConfigurationWindowLoadData (config_dlg.cwn, config_dlg)
	config_dlg.isLoading = false
	Enemy.CombatLogUI_ConfigDialog_TestSettings ()
end


function Enemy.CombatLogUI_ConfigDialog_OnReset (section)

	for k, v in pairs (Enemy.Settings)
	do
		if (not k:match("^combatLog.*")) then continue end
		Enemy.Settings[k] = nil
	end
	
	Enemy.Print ("Combat log settings has been reset")
	InterfaceCore.ReloadUI ()
end


function Enemy.CombatLogUI_ConfigDialog_TestSettings ()

	if (config_dlg.isLoading) then return end
	
	Enemy.ConfigurationWindowSaveData (config_dlg.cwn, config_dlg)
	Enemy.CombatLog_OnSettingsChanged (config_dlg)
end


function Enemy.CombatLogUI_ConfigDialog_OnSave (section)

	Enemy.CombatLogUI_ConfigDialog_TestSettings ()
	
	Enemy.Settings.combatLogEnabled = config_dlg.combatLogEnabled
	Enemy.Settings.combatLogLogParseErrors = config_dlg.combatLogLogParseErrors
	Enemy.Settings.combatLogStatisticsEnabled = config_dlg.combatLogStatisticsEnabled
	
--	Enemy.Settings.combatLogCurrentStatisticsEnabled =
--	{
--		[Enemy.CombatLogTypes.OutgoingDamage] = config_dlg.combatLogCurrentOutgoingDamageStatisticsEnabled,
--		[Enemy.CombatLogTypes.OutgoingHeal] = config_dlg.combatLogCurrentOutgoingHealStatisticsEnabled,
--		[Enemy.CombatLogTypes.IncommingDamage] = config_dlg.combatLogCurrentIncommingDamageStatisticsEnabled,
--		[Enemy.CombatLogTypes.IncommingHeal] = config_dlg.combatLogCurrentIncommingHealStatisticsEnabled
--	}
--	Enemy.Settings.combatLogCurrentStatisticsTimeframe = config_dlg.combatLogCurrentStatisticsTimeframe

	Enemy.Settings.combatLogIDSEnabled = config_dlg.combatLogIDSEnabled
	Enemy.Settings.combatLogIDSType = config_dlg.combatLogIDSType
	Enemy.Settings.combatLogIDSTimeframe = config_dlg.combatLogIDSTimeframe
	Enemy.Settings.combatLogIDSMaxRows = config_dlg.combatLogIDSMaxRows
	Enemy.Settings.combatLogIDSDisplayTime = config_dlg.combatLogIDSDisplayTime
	Enemy.Settings.combatLogIDSRowSize = Enemy.clone (config_dlg.combatLogIDSRowSize)
	Enemy.Settings.combatLogIDSRowPadding = config_dlg.combatLogIDSRowPadding
	Enemy.Settings.combatLogIDSRowScale = config_dlg.combatLogIDSRowScale
	Enemy.Settings.combatLogIDSRowBackground = Enemy.clone (config_dlg.combatLogIDSRowBackground)
	Enemy.Settings.combatLogIDSRowBackgroundAlpha = config_dlg.combatLogIDSRowBackgroundAlpha
	
	return true
end




