local Enemy = Enemy
local dlg = nil
local g = nil

local tinsert = table.insert
local tsort = table.sort
local pairs = pairs
local ipairs = ipairs
local mfloor = math.floor
local unpack = unpack


function Enemy.CombatLogUI_StatsWindow_Initialize ()

	g = Enemy.combatLog
	
	Enemy.AddEventHandler ("CombatLogUI_StatsWindow", "IconCreateContextMenu", function (data)
		
		if (Enemy.CombatLogUI_StatsWindow_IsOpen ())
		then
			table.insert (data,
				{
					text = L"Hide combat log stats window",
					callback = Enemy.CombatLogUI_StatsWindow_Hide
				})
		else
			Enemy.ScenarioInfoUpdateData ()

			table.insert (data,
				{
					text = L"Show combat log stats window",
					callback = Enemy.CombatLogUI_StatsWindow_Open,
					disabled = (not Enemy.Settings.combatLogEnabled or not Enemy.Settings.combatLogStatisticsEnabled)
				})
		end
	end)
	
end


function Enemy.CombatLogUI_StatsWindow_IsOpen ()
	return dlg and WindowGetShowing ("EnemyCombatLogStatsWindow")
end


function Enemy.CombatLogUI_StatsWindow_Hide ()

	WindowSetShowing ("EnemyCombatLogStatsWindow", false)
	
	if (dlg)
	then
		Enemy.RemoveEventHandler ("CombatLogUI_StatsWindow", "CombatLogSessionsUpdated")
		Enemy.RemoveEventHandler ("CombatLogUI_StatsWindow", "CombatLogNewCombatEvent")
		
		dlg.isActive = false
	end
end


function Enemy.CombatLogUI_StatsWindow_Open ()

	if (not g.settings.combatLogEnabled or not g.settings.combatLogStatisticsEnabled) then return end

	if (not dlg)
	then
		-- initialize
		CreateWindow ("EnemyCombatLogStatsWindow", false)
		LabelSetText ("EnemyCombatLogStatsWindowTitleBarText", L"Combat log statistics")
		
		LabelSetText ("EnemyCombatLogStatsWindowSessionLabel", L"Session")
		ButtonSetText ("EnemyCombatLogStatsWindowSessionAddButton", L"New")
		LabelSetText ("EnemyCombatLogStatsWindowTypeLabel", L"Type")
		
		dlg = {}

		dlg.types =
		{
			[1] =
			{
				text = L"Outgoing damage",
				value = Enemy.CombatLogTypes.OutgoingDamage,
				defaultColumns = { "name", "totalPercent", "critPercent", "avg", "max", "mitPercent" },
				defaultSortColumn = 2,
				defaultSortDirection = false
			},
			
			[2] =
			{
				text = L"Outgoing heal",
				value = Enemy.CombatLogTypes.OutgoingHeal,
				defaultColumns = { "name", "totalPercent", "critPercent", "avg", "max", "overhealPercent" },
				defaultSortColumn = 2,
				defaultSortDirection = false
			},
			
			[3] =
			{
				text = L"Incomming damage",
				value = Enemy.CombatLogTypes.IncommingDamage,
				defaultColumns = { "name", "totalPercent", "critPercent", "avg", "max", "mitPercent" },
				defaultSortColumn = 2,
				defaultSortDirection = false
			},
			
			[4] =
			{
				text = L"Incomming heal",
				value = Enemy.CombatLogTypes.IncommingHeal,
				defaultColumns = { "name", "totalPercent", "critPercent", "avg", "max", "overhealPercent" },
				defaultSortColumn = 2,
				defaultSortDirection = false
			}
		}
		
		dlg.columns =
		{
			name =
			{
				key = "name",
				name = L"Name",
				title = L"Name",
				getValueString = function (item)
					return item.name
				end,
				types =
				{
					[Enemy.CombatLogTypes.OutgoingDamage] = false,
					[Enemy.CombatLogTypes.OutgoingHeal] = false,
					[Enemy.CombatLogTypes.IncommingDamage] = false,
					[Enemy.CombatLogTypes.IncommingHeal] = false
				},
				defaultSortDirection = true,
				order = 5
			},
			
			count =
			{
				key = "count",
				name = L"Count",
				title = L"Count",
				getValueString = function (item)
					return Enemy.FormatNumberShort (item.count, true)
				end,
				color = { 150, 150, 150 },
				types =
				{
					[Enemy.CombatLogTypes.OutgoingDamage] = true,
					[Enemy.CombatLogTypes.OutgoingHeal] = true,
					[Enemy.CombatLogTypes.IncommingDamage] = true,
					[Enemy.CombatLogTypes.IncommingHeal] = true
				},
				defaultSortDirection = false,
				order = 10
			},
			
			critPercent =
			{
				key = "critPercent",
				name = L"Crit %",
				title = L"Crit",
				getValueString = function (item)
					return Enemy.toWString (item.critPercent)..L" %"
				end,
				color = { 255, 75, 75 },
				isPercent = true,
				types =
				{
					[Enemy.CombatLogTypes.OutgoingDamage] = true,
					[Enemy.CombatLogTypes.OutgoingHeal] = true,
					[Enemy.CombatLogTypes.IncommingDamage] = true,
					[Enemy.CombatLogTypes.IncommingHeal] = true
				},
				defaultSortDirection = false,
				order = 20
			},
			
			min =
			{
				key = "min",
				name = L"Min",
				title = L"Min",
				getValueString = function (item)
					return Enemy.FormatNumberShort (item.min, true)
				end,
				types =
				{
					[Enemy.CombatLogTypes.OutgoingDamage] = true,
					[Enemy.CombatLogTypes.OutgoingHeal] = true,
					[Enemy.CombatLogTypes.IncommingDamage] = true,
					[Enemy.CombatLogTypes.IncommingHeal] = true
				},
				defaultSortDirection = false,
				order = 30
			},
			avg =
			{
				key = "avg",
				name = L"Average",
				title = L"Average",
				getValueString = function (item)
					return Enemy.FormatNumberShort (item.avg, true)
				end,
				types =
				{
					[Enemy.CombatLogTypes.OutgoingDamage] = true,
					[Enemy.CombatLogTypes.OutgoingHeal] = true,
					[Enemy.CombatLogTypes.IncommingDamage] = true,
					[Enemy.CombatLogTypes.IncommingHeal] = true
				},
				defaultSortDirection = false,
				order = 40
			},
			max =
			{
				key = "max",
				name = L"Max",
				title = L"Max",
				getValueString = function (item)
					return Enemy.FormatNumberShort (item.max, true)
				end,
				types =
				{
					[Enemy.CombatLogTypes.OutgoingDamage] = true,
					[Enemy.CombatLogTypes.OutgoingHeal] = true,
					[Enemy.CombatLogTypes.IncommingDamage] = true,
					[Enemy.CombatLogTypes.IncommingHeal] = true
				},
				defaultSortDirection = false,
				order = 50
			},
			
			total =
			{
				key = "total",
				name = L"Total",
				title = L"Total",
				getValueString = function (item)
					return Enemy.FormatNumberShort (item.total, true)
				end,
				color = { 255, 225, 75 },
				types =
				{
					[Enemy.CombatLogTypes.OutgoingDamage] = true,
					[Enemy.CombatLogTypes.OutgoingHeal] = true,
					[Enemy.CombatLogTypes.IncommingDamage] = true,
					[Enemy.CombatLogTypes.IncommingHeal] = true
				},
				defaultSortDirection = false,
				order = 60
			},
			totalPercent =
			{
				key = "totalPercent",
				name = L"Total %",
				title = L"Total",
				getValueString = function (item)
					return Enemy.toWString (item.totalPercent)..L" %"
				end,
				color = { 255, 225, 75 },
				isPercent = true,
				types =
				{
					[Enemy.CombatLogTypes.OutgoingDamage] = true,
					[Enemy.CombatLogTypes.OutgoingHeal] = true,
					[Enemy.CombatLogTypes.IncommingDamage] = true,
					[Enemy.CombatLogTypes.IncommingHeal] = true
				},
				defaultSortDirection = false,
				order = 65
			},
			totalRaw =
			{
				key = "totalRaw",
				name = L"Total raw",
				title = L"Total raw",
				getValueString = function (item)
					return Enemy.FormatNumberShort (item.totalRaw, true)
				end,
				color = { 255, 225, 75 },
				types =
				{
					[Enemy.CombatLogTypes.OutgoingDamage] = true,
					[Enemy.CombatLogTypes.OutgoingHeal] = false,
					[Enemy.CombatLogTypes.IncommingDamage] = true,
					[Enemy.CombatLogTypes.IncommingHeal] = false
				},
				defaultSortDirection = false,
				order = 70
			},
						
			totalWithOverheal =
			{
				key = "totalWithOverheal",
				name = L"Total with overheal",
				title = L"Total WO",
				getValueString = function (item)
					return Enemy.FormatNumberShort (item.totalWithOverheal, true)
				end,
				color = { 155, 255, 75 },
				types =
				{
					[Enemy.CombatLogTypes.OutgoingDamage] = false,
					[Enemy.CombatLogTypes.OutgoingHeal] = true,
					[Enemy.CombatLogTypes.IncommingDamage] = false,
					[Enemy.CombatLogTypes.IncommingHeal] = true
				},
				defaultSortDirection = false,
				order = 80
			},
			totalWithOverhealPercent =
			{
				key = "totalWithOverhealPercent",
				name = L"Total with overheal %",
				title = L"Total WO",
				getValueString = function (item)
					return Enemy.toWString (item.totalWithOverhealPercent)..L" %"
				end,
				color = { 155, 255, 75 },
				isPercent = true,
				types =
				{
					[Enemy.CombatLogTypes.OutgoingDamage] = false,
					[Enemy.CombatLogTypes.OutgoingHeal] = true,
					[Enemy.CombatLogTypes.IncommingDamage] = false,
					[Enemy.CombatLogTypes.IncommingHeal] = true
				},
				defaultSortDirection = false,
				order = 90
			},
			overhealPercent =
			{
				key = "overhealPercent",
				name = L"Overheal %",
				title = L"Overheal",
				getValueString = function (item)
					return Enemy.toWString (item.overhealPercent)..L" %"
				end,
				color = { 155, 255, 75 },
				isPercent = true,
				types =
				{
					[Enemy.CombatLogTypes.OutgoingDamage] = false,
					[Enemy.CombatLogTypes.OutgoingHeal] = true,
					[Enemy.CombatLogTypes.IncommingDamage] = false,
					[Enemy.CombatLogTypes.IncommingHeal] = true
				},
				defaultSortDirection = false,
				order = 100
			},
			
			mit =
			{
				key = "mit",
				name = L"Mitigation",
				title = L"Mitigation",
				getValueString = function (item)
					return Enemy.FormatNumberShort (item.mit, true)
				end,
				color = { 75, 145, 255 },
				types =
				{
					[Enemy.CombatLogTypes.OutgoingDamage] = true,
					[Enemy.CombatLogTypes.OutgoingHeal] = false,
					[Enemy.CombatLogTypes.IncommingDamage] = true,
					[Enemy.CombatLogTypes.IncommingHeal] = false
				},
				defaultSortDirection = false,
				order = 110
			},
			mitPercent =
			{
				key = "mitPercent",
				name = L"Mitigation %",
				title = L"Mitigation",
				getValueString = function (item)
					return Enemy.toWString (item.mitPercent)..L" %"
				end,
				color = { 75, 145, 255 },
				isPercent = true,
				types =
				{
					[Enemy.CombatLogTypes.OutgoingDamage] = true,
					[Enemy.CombatLogTypes.OutgoingHeal] = false,
					[Enemy.CombatLogTypes.IncommingDamage] = true,
					[Enemy.CombatLogTypes.IncommingHeal] = false
				},
				defaultSortDirection = false,
				order = 120
			},
			abs =
			{
				key = "abs",
				name = L"Absorb",
				title = L"Absorb",
				getValueString = function (item)
					return Enemy.FormatNumberShort (item.abs, true)
				end,
				color = { 75, 145, 255 },
				types =
				{
					[Enemy.CombatLogTypes.OutgoingDamage] = true,
					[Enemy.CombatLogTypes.OutgoingHeal] = false,
					[Enemy.CombatLogTypes.IncommingDamage] = true,
					[Enemy.CombatLogTypes.IncommingHeal] = false
				},
				defaultSortDirection = false,
				order = 130
			},
			absPercent =
			{
				key = "absPercent",
				name = L"Absorb %",
				title = L"Absorb",
				getValueString = function (item)
					return Enemy.toWString (item.absPercent)..L" %"
				end,
				isPercent = true,
				color = { 75, 145, 255 },
				types =
				{
					[Enemy.CombatLogTypes.OutgoingDamage] = true,
					[Enemy.CombatLogTypes.OutgoingHeal] = false,
					[Enemy.CombatLogTypes.IncommingDamage] = true,
					[Enemy.CombatLogTypes.IncommingHeal] = false
				},
				defaultSortDirection = false,
				order = 140
			},
			
			blockPercent =
			{
				key = "blockPercent",
				name = L"Block %",
				title = L"Block",
				getValueString = function (item)
					return Enemy.toWString (item.blockPercent)..L" %"
				end,
				isPercent = true,
				color = { 225, 145, 235 },
				types =
				{
					[Enemy.CombatLogTypes.OutgoingDamage] = true,
					[Enemy.CombatLogTypes.OutgoingHeal] = false,
					[Enemy.CombatLogTypes.IncommingDamage] = true,
					[Enemy.CombatLogTypes.IncommingHeal] = false
				},
				defaultSortDirection = false,
				order = 145
			},
			parryPercent =
			{
				key = "parryPercent",
				name = L"Parry %",
				title = L"Parry",
				getValueString = function (item)
					return Enemy.toWString (item.parryPercent)..L" %"
				end,
				isPercent = true,
				color = { 225, 145, 235 },
				types =
				{
					[Enemy.CombatLogTypes.OutgoingDamage] = true,
					[Enemy.CombatLogTypes.OutgoingHeal] = false,
					[Enemy.CombatLogTypes.IncommingDamage] = true,
					[Enemy.CombatLogTypes.IncommingHeal] = false
				},
				defaultSortDirection = false,
				order = 150
			},
			disruptPercent =
			{
				key = "disruptPercent",
				name = L"Disrupt %",
				title = L"Disrupt",
				getValueString = function (item)
					return Enemy.toWString (item.disruptPercent)..L" %"
				end,
				isPercent = true,
				color = { 225, 145, 235 },
				types =
				{
					[Enemy.CombatLogTypes.OutgoingDamage] = true,
					[Enemy.CombatLogTypes.OutgoingHeal] = false,
					[Enemy.CombatLogTypes.IncommingDamage] = true,
					[Enemy.CombatLogTypes.IncommingHeal] = false
				},
				defaultSortDirection = false,
				order = 160
			},
			dodgePercent =
			{
				key = "dodgePercent",
				name = L"Dodge %",
				title = L"Dodge",
				getValueString = function (item)
					return Enemy.toWString (item.dodgePercent)..L" %"
				end,
				isPercent = true,
				color = { 225, 145, 235 },
				types =
				{
					[Enemy.CombatLogTypes.OutgoingDamage] = true,
					[Enemy.CombatLogTypes.OutgoingHeal] = false,
					[Enemy.CombatLogTypes.IncommingDamage] = true,
					[Enemy.CombatLogTypes.IncommingHeal] = false
				},
				defaultSortDirection = false,
				order = 170
			 }
		}
		
		dlg.columnsSorted = {}
		for _, column in pairs (dlg.columns)
		do
			tinsert (dlg.columnsSorted, column)
		end
		
		tsort (dlg.columnsSorted, function (a, b)
			return a.order < b.order
		end)
		
		for _, t in ipairs (dlg.types)
		do
			ComboBoxAddMenuItem ("EnemyCombatLogStatsWindowType", t.text)
		end

		ComboBoxSetSelectedMenuItem ("EnemyCombatLogStatsWindowType", g.settings.combatLogStatsWindowType or 1)
		dlg.initialized = true
	end

	WindowSetShowing ("EnemyCombatLogStatsWindow", true)
	
	Enemy.AddEventHandler ("CombatLogUI_StatsWindow", "CombatLogSessionsUpdated", Enemy.CombatLogUI_StatsWindow_UpdateSessionsList)
	Enemy.AddEventHandler ("CombatLogUI_StatsWindow", "CombatLogNewCombatEvent", Enemy.CombatLogUI_StatsWindow_OnNewCombatEvent)
	dlg.isActive = true
	
	Enemy.CombatLogUI_StatsWindow_UpdateSessionsList ()
end


function Enemy.CombatLogUI_StatsWindow_UpdateSessionsList ()

	if (not dlg.initialized) then return end

	ComboBoxClearMenuItems ("EnemyCombatLogStatsWindowSession")
	dlg.sessions = {}
		
	for _, stats in ipairs (g.allStats)
	do
		local session =
		{
			stats = stats,
			order = stats.dt.totalSeconds
		}
		
		session.name = Enemy.DateTimeToString (stats.dt)
		if (stats.name) then session.name = session.name..L" - "..stats.name end
		
		tinsert (dlg.sessions, session)
	end
	
	tsort (dlg.sessions, function (a, b)
		return a.order < b.order
	end)
	
	local index = #dlg.sessions
	for k, v in ipairs (dlg.sessions)
	do
		ComboBoxAddMenuItem ("EnemyCombatLogStatsWindowSession", v.name)
		if (v.stats == g.stats)
		then
			index = k
		end
	end
	
	ComboBoxSetSelectedMenuItem ("EnemyCombatLogStatsWindowSession", index)
	Enemy.CombatLogUI_StatsWindow_OnSessionSelChanged ()
end


function Enemy.CombatLogUI_StatsWindow_GetCurrentSession ()
	return dlg.sessions[ComboBoxGetSelectedMenuItem ("EnemyCombatLogStatsWindowSession")]
end


function Enemy.CombatLogUI_StatsWindow_OnSessionSelChanged ()

	if (not dlg.initialized) then return end

	local session = Enemy.CombatLogUI_StatsWindow_GetCurrentSession ()
	if (not session) then return end
	
	if (session.stats == g.stats)
	then
		ButtonSetText ("EnemyCombatLogStatsWindowSessionDeleteButton", L"Reset")
	else
		ButtonSetText ("EnemyCombatLogStatsWindowSessionDeleteButton", L"Delete")
	end
	
	Enemy.CombatLogUI_StatsWindow_OnTypeSelChanged ()
end


function Enemy.CombatLogUI_StatsWindow_SessionAdd ()

	DialogManager.MakeTextEntryDialog (L"New combat log session", L"Enter session name", L"Default", function (text)
		Enemy.CombatLog_StatsSessionAdd (Enemy.trim (text))
	end,
	nil,
	250,
	false)
end


function Enemy.CombatLogUI_StatsWindow_SessionDelete ()

	local session = Enemy.CombatLogUI_StatsWindow_GetCurrentSession ()
	if (not session) then return end

	if (session.stats == g.stats)
	then
		DialogManager.MakeTwoButtonDialog (L"Reset session '"..session.name..L"' ?",
	                                       L"Yes", function ()
	                                       		Enemy.CombatLog_ResetStatsCurrentSession ()
	                                       end,
	                                       L"No")
	else
		DialogManager.MakeTwoButtonDialog (L"Delete session '"..session.name..L"' ?",
	                                       L"Yes", function ()
	                                       		Enemy.CombatLog_StatsSessionDelete (session.stats)
	                                       end,
	                                       L"No")
	end
end


function Enemy.CombatLogUI_StatsWindow_GetCurrentType ()
	return dlg.types[ComboBoxGetSelectedMenuItem ("EnemyCombatLogStatsWindowType")]
end


function Enemy.CombatLogUI_StatsWindow_OnTypeSelChanged ()

	if (not dlg.initialized) then return end

	local t = Enemy.CombatLogUI_StatsWindow_GetCurrentType ()
	if (not t) then return end
	
	g.settings.combatLogStatsWindowType = ComboBoxGetSelectedMenuItem ("EnemyCombatLogStatsWindowType")
	
	-- prepare columns
	local columns_settings = g.settings.combatLogStatsWindowColumns
	if (not columns_settings)
	then
		columns_settings = {}
		g.settings.combatLogStatsWindowColumns = columns_settings
	end
	
	dlg.column_settings = columns_settings[t.value]
	if (not dlg.column_settings)
	then
		dlg.column_settings =
		{
			sortColumn = t.defaultSortColumn,
			sortDirection = t.defaultSortDirection,
			columns = Enemy.clone (t.defaultColumns)
		}
		columns_settings[t.value] = dlg.column_settings
	end
	
	Enemy.CombatLogUI_StatsWindow_UpdateColumns ()
	Enemy.CombatLogUI_StatsWindow_UpdateList ()
end


function Enemy.CombatLogUI_StatsWindow_OnNewCombatEvent (combatEvent)
	
	local t = Enemy.CombatLogUI_StatsWindow_GetCurrentType ()
	if (not t or combatEvent.type ~= t.value) then return end
	
	Enemy.CombatLogUI_StatsWindow_UpdateList ()
end


function Enemy.CombatLogUI_StatsWindow_UpdateColumns ()

	local t = Enemy.CombatLogUI_StatsWindow_GetCurrentType ()
	if (not t) then return end
	
	for k, v in ipairs (dlg.column_settings.columns)
	do
		local wn = "EnemyCombatLogStatsWindowListHeaderSortColumn"..k
		local column_info = dlg.columns[v]
		
		ButtonSetText (wn, column_info.title)		
		WindowSetShowing (wn.."UpArrow", k == dlg.column_settings.sortColumn and dlg.column_settings.sortDirection)
		WindowSetShowing (wn.."DownArrow", k == dlg.column_settings.sortColumn and not dlg.column_settings.sortDirection)
	end
end


function Enemy.CombatLogUI_StatsWindow_UpdateList ()

	local session = Enemy.CombatLogUI_StatsWindow_GetCurrentSession ()
	if (not session) then return end
	
	local t = Enemy.CombatLogUI_StatsWindow_GetCurrentType ()
	local stats_data = session.stats.data[t.value] or {}
	
	-- prepare list data
	local data = {}
	local total_sum = 0
	local total_with_overheal_sum = 0
	
	for k, v in pairs (stats_data)
	do
		local item =
		{
			name = Enemy.capitalize (k),
			count = v.countNormal + v.countCrits,
			total = v.totalNormal + v.totalCrits,
			mit = v.mit,
			abs = v.abs,
			min = v.min,
			max = v.max
		}
		
		if (item.count == 0) then continue end
		
		item.critPercent = mfloor (v.countCrits / item.count * 100.0 + 0.5)
		
		item.totalWithOverheal = item.total + v.totalOverheal
		
		if (item.totalWithOverheal == 0)
		then
			item.overhealPercent = 0
		else
			item.overhealPercent = mfloor (v.totalOverheal / item.totalWithOverheal * 100.0 + 0.5)
		end
		
		item.totalRaw = item.total + item.mit + item.abs
		
		if (item.totalRaw == 0)
		then
--			if (item.totalWithOverheal == 0)
--			then
--				continue
--			end
			 
			item.mitPercent = 0
			item.absPercent = 0
		else
			item.mitPercent = mfloor (item.mit / item.totalRaw * 100.0 + 0.5)
			item.absPercent = mfloor (item.abs / item.totalRaw * 100.0 + 0.5)
		end
		
		item.avg = mfloor (0.5 * (item.min + item.max) + 0.5)
		
		item.blockPercent = mfloor (v.block / item.count * 100.0 + 0.5)
		item.parryPercent = mfloor (v.parry / item.count * 100.0 + 0.5)
		item.disruptPercent = mfloor (v.disrupt / item.count * 100.0 + 0.5)
		item.dodgePercent = mfloor (v.dodge / item.count * 100.0 + 0.5)
		
		total_sum = total_sum + item.total
		total_with_overheal_sum = total_with_overheal_sum + item.totalWithOverheal
		
		tinsert (data, item)
	end
	
	for _, item in ipairs (data)
	do
		if (total_sum == 0)
		then
			item.totalPercent = 0
		else
			item.totalPercent = mfloor (item.total / total_sum * 100.0 + 0.5)
		end
		
		if (total_with_overheal_sum == 0)
		then
			item.totalWithOverhealPercent = 0
		else
			item.totalWithOverhealPercent = mfloor (item.totalWithOverheal / total_with_overheal_sum * 100.0 + 0.5)
		end
	end
	
	-- sorting
	local sort_column = dlg.column_settings.columns[dlg.column_settings.sortColumn]
	
	tsort (data, function (a, b)
		return a[sort_column] < b[sort_column]
	end)
	
	local indexes = {}
	local max = #data
	for index, _ in ipairs (data)
	do
		if (dlg.column_settings.sortDirection)
		then
			tinsert (indexes, index)
		else
			tinsert (indexes, max - index + 1)
		end
	end
	
	Enemy.combatLogStatsWindowListData = data
	Enemy.combatLogStatsWindowListIndexes = indexes
	Enemy.CombatLogUI_StatsWindow_RedrawList ()
end


function Enemy.CombatLogUI_StatsWindow_RedrawList ()
	ListBoxSetDisplayOrder ("EnemyCombatLogStatsWindowList", Enemy.combatLogStatsWindowListIndexes)
end


function Enemy.CombatLogUI_StatsWindow_OnListPopulate ()
	
	local data = Enemy.combatLogStatsWindowListData
	
	for k, index in ipairs (EnemyCombatLogStatsWindowList.PopulatorIndices)
	do
		local wnr = "EnemyCombatLogStatsWindowListRow"..k
		local item = data[index]
		
		for column_index, v in ipairs (dlg.column_settings.columns)
		do
			LabelSetText (wnr.."Value"..column_index, dlg.columns[v].getValueString (item))
		end

		local wnr_background = wnr.."Background"
				
		if (index == dlg.hoverRowIndex)
		then
			WindowSetTintColor (wnr_background, 255, 204, 102)
			WindowSetAlpha (wnr_background, 0.4)
			WindowSetShowing (wnr_background, true)
		else
			WindowSetShowing (wnr_background, false)
		end
	end
end


function Enemy.CombatLogUI_StatsWindow_SortColumnClick ()

	local sort_column = SystemData.ActiveWindow.name:match ("^EnemyCombatLogStatsWindowListHeaderSortColumn(%d+)$")
	if (not sort_column) then return end
	sort_column = tonumber (sort_column)
	
	if (dlg.column_settings.sortColumn ~= sort_column)
	then
		dlg.column_settings.sortColumn = sort_column
		
		local column_info = dlg.columns[dlg.column_settings.columns[dlg.column_settings.sortColumn]]
		dlg.column_settings.sortDirection = column_info.defaultSortDirection
	else
		dlg.column_settings.sortDirection = not dlg.column_settings.sortDirection
	end
	
	Enemy.CombatLogUI_StatsWindow_UpdateColumns ()
	Enemy.CombatLogUI_StatsWindow_UpdateList ()
end


function Enemy.CombatLogUI_StatsWindow_SortColumnRClick ()

	local sort_column = SystemData.ActiveWindow.name:match ("^EnemyCombatLogStatsWindowListHeaderSortColumn(%d+)$")
	if (not sort_column) then return end
	sort_column = tonumber (sort_column)
	
	local t = Enemy.CombatLogUI_StatsWindow_GetCurrentType ()
	if (not t) then return end
	
	EA_Window_ContextMenu.CreateContextMenu (SystemData.ActiveWindow.name, 1)
	WindowSetLayer ("EA_Window_ContextMenu1", Window.Layers.OVERLAY)
	
	for _, column_info in ipairs (dlg.columnsSorted)
	do
		if (not column_info.types[t.value]) then continue end
	
		EA_Window_ContextMenu.AddMenuItem (column_info.name, function ()
		
			dlg.column_settings.columns[sort_column] = column_info.key
			
			Enemy.CombatLogUI_StatsWindow_UpdateColumns ()
			Enemy.CombatLogUI_StatsWindow_UpdateList ()
			
		end, false, true)
	end
	
	EA_Window_ContextMenu.Finalize ()
end


function Enemy.UCombatLogUI_StatsWindow_ListRowMouseOver ()

	local row_index = SystemData.ActiveWindow.name:match ("^EnemyCombatLogStatsWindowListRow(%d+)Value%d+$")
	if (not row_index)
	then
		dlg.hoverRowIndex = nil
		return
	end
	
	row_index = ListBoxGetDataIndex ("EnemyCombatLogStatsWindowList", tonumber (row_index))
	
	local t = Enemy.CombatLogUI_StatsWindow_GetCurrentType ()
	if (not t) then return end
	
	dlg.hoverRowIndex = row_index
	Enemy.CombatLogUI_StatsWindow_RedrawList ()
	
	local item = Enemy.combatLogStatsWindowListData[dlg.hoverRowIndex]
	
	if (not item.tooltip)
	then
		item.tooltip =
		{
			{ text = item.name }
		}
		
		for _, column_info in ipairs (dlg.columnsSorted)
		do
			if (not column_info.types[t.value]) then continue end
			
			tinsert (item.tooltip,
			{
				text = column_info.name,
				text2 = column_info.getValueString (item),
				color = column_info.color
			})
		end
	end
	
	Tooltips.CreateTextOnlyTooltip (SystemData.MouseOverWindow.name)
	
	for k, tp in ipairs (item.tooltip)
	do
		Tooltips.SetTooltipText (k, 1, tp.text)
		Tooltips.SetTooltipText (k, 2, L"           ")
		Tooltips.SetTooltipText (k, 3, Enemy.isNil (tp.text2, L""))
		
		if (tp.color)
		then
			local r, g, b = unpack (tp.color)
			Tooltips.SetTooltipColor (k, 1, r, g, b)
			Tooltips.SetTooltipColor (k, 3, r, g, b)
		end
	end
	
	Tooltips.AnchorTooltip (Tooltips.ANCHOR_CURSOR)
	Tooltips.Finalize()
end
















